## Mathematikinformation Nr. 71 / 72
## http://mathematikinformation.info/
##
## Lei Baozhen, Beijing Union University
## Harald Loewe, Technische Universitaet Braunschweig
## Numerische Verfahren zur Loesung von Differentialgleichungen
##
##############################################################
### Zu Aufgabe 1.7.5 (Logistisches Wachstum)               ###
##############################################################
##
## Ver. 2019-12-15
## Kodierung in UTF-8; Octave-Version 5.10

## Initialisierung des Anfangswertproblems
## Aufraeumen
clear -all;
clf;

###########################################
## Parameter fuer das Script             ##
###########################################
r     = 0.0017;       # Wachstumsrate
K     = 10000;        # Grenzbestand
dt    = 0.1;          # Schrittweite fuer ODE_PZ
alf   = 10^-1;        # Akzeptabler lokaler Fehler fuer ODE_PZSW

## Anfangswertproblem
t0    = 0;            # Intervallanfang
T     = 2;            # Intervallende
f     = @(t,x) r*x*(K-x); # rechte Seite der Differentialgleichung
x0    = 3;            # Anfangsbestand
## Exakte Loesung und deren Graph
xe    = @(t) K./(1+(K-x0)/x0*exp(-r*K*t));
tte   = 0:0.01:T;

## Polygonzugverfahren mit und ohne Schrittweitensteuerung
erg_oSW = ODE_PZ(f, t0, T, x0, dt);
erg_mSW = ODE_RKSW(f, t0, T, x0,{'alpha',alf});


fig = figure(1);
groesse = [15 10];
set (fig,'papersize',groesse)
set (fig, 'paperposition', [0,0,groesse]);
    ## Plot der exakten Loesung in Grau
    plot(tte,xe(tte),'linewidth',6,'color',[.7 .7 .7])
    hold on                 # naechste Grafik dazufuegen
    ## Plot der Naeherungsloesungen
    plot(erg_oSW(1,:),erg_oSW(2,:),'linewidth',2,'color','red')
    plot(erg_mSW(1,:),erg_mSW(2,:),'linewidth',2,'color','blue')
    xlabel('t'), ylabel('x(t)'), axis([t0 T])
    title(['Logistisches Wachstum: r = ',num2str(r),', K = ',num2str(K);...
           'ODE\_PZ (rot) mit dt = ',num2str(dt),...
           ';  ODE\_PZSW (blau) mit \alpha = ',num2str(alf)])
    set(gca,'linewidth',2,'fontsize',18)
    hold off

## Abspeichern als Bild in akzeptabler Aufloesung.
print ("-r600", "-painters", ["Aufgabe_1_7_5.png"]);