## Mathematikinformation Nr. 71 / 72
## http://mathematikinformation.info/
##
## Lei Baozhen, Beijing Union University
## Harald Loewe, Technische Universitaet Braunschweig
## Numerische Verfahren zur Loesung von Differentialgleichungen
##
##############################################################
### Zu Aufgabe 2.2.2 (Crank-Nicholson-Verfahren)           ###
##############################################################
##
## Ver. 2019-12-15
## Kodierung in UTF-8; Octave-Version 5.10

## Aufraeumen
clear -all;
clf;

##############################################################
## Parameterwahl                                           ###
## Zerfallskonstanze kappa                                 ###
## Schrittweite dt                                         ###
##############################################################
kp  = 0.1824;               # Zerfallskonstante
dt  = 10;                   # Schrittweite
## Fuer die Zerfallskonstante 0.1824 von Radon ist
## 2/kappa = 10.965 und 1/kappa = 5.4825
## Fuer Schrittweiten zwischen 6 und 10 erwarten wir daher eine
## monoton abklingende Loesung fuer das Crank-Nicholson-Verfahren
## sowie eine unsinnige Loesung fuer das Polygonzugverfahren


## Anfangswertproblem fuer den Zerfall von Radon:
f   = @(t,x) -kp*x;         # rechte Seite
t0  = 0;                    # Intervall-Anfang
T   = 365;                  # Intervall-Ende (365 Tage)
x0  = 10^3;                 # Anfangswert    (1000 kg)

## Exakte Loesung:
xe  = @(t) x0*exp(-kp*t);   # Exakte Loesung als Funktion
tte = [0:1:T];              # Stuetzstellen fuer exakte Loesung

## Erzeugen der Stuetzstellen
ttt = 0:dt:T;
anz = length(ttt);

## Diesmal nehmen wir die expliziten Gleichungen
## Zunaechst das Polygonzugverfahren
xxxPZ = x0*(1-kp*dt).^[0:1:anz-1];
## Dann fuer das Crank-Nicho�son-Verfahren
xxxCN = x0*((2-kp*dt)/(2+kp*dt)).^[0:1:anz-1];

fig = figure(1);
groesse = [10 10];
set (fig,'papersize',groesse)
set (fig, 'paperposition', [0,0,groesse]);
    ## Plot der exakten Loesung in Grau
    plot(tte,xe(tte),'linewidth',2,'color',[.5 .5 .5])
    hold on                 # naechste Grafik dazufuegen
    ## Plot des Polygonzugs
    plot(ttt,xxxPZ,'linewidth',3,'color','red')
    ## Plot von Crank-Nicholson
    plot(ttt,xxxCN,'linewidth',3,'color','blue')
    ## Legende, Label etc.
    xlabel('Zeit'), ylabel('Bestand'), axis([t0 T])
    title(['dt = ',num2str(dt)])
    leg = legend('x(t)', 'x_k^{PZ}', 'x_k^{CN}');
    set(leg,'fontsize',18)
    set(gca,'linewidth',2,'fontsize',24)
    hold off