## Mathematikinformation Nr. 71 / 72
## http://mathematikinformation.info/
##
## Lei Baozhen, Beijing Union University
## Harald Loewe, Technische Universitaet Braunschweig
## Numerische Verfahren zur Loesung von Differentialgleichungen
##
##############################################################
### Zu Aufgabe 2.2.3                                       ###
##############################################################
##
## Ver. 2019-12-15
## Kodierung in UTF-8; Octave-Version 5.10

## Aufraeumen
clear -a;
clf;

## Festlegen der Parameter und der Schrittweite
r     = 0.001;        # Wachstumsrate
K     = 10000;        # Grenzbestand
t0    = 0;            # Intervallanfang
T     = 2;            # Intervallende

## Festlegung des Anfangswertproblems
f     = @(t,x) r*x*(K-x);              # rechte Seite der Differentialgleichung
x0    = 100;                           # Anfangswert

## Exakte Loesung
x     = @(t) K./(1+(K-x0)/x0*exp(-r*K*t));
tte   = 0:0.01:T;

################################################
### Festlegen der Schrittweite dt            ###
################################################
dt_PZ = 0.01;
dt_EH = 0.06;

## Durchfuehren der beiden Verfahren
tic;
erg_PZ   = ODE_PZ(f, t0, T, x0, dt_PZ); # Polygonzugverfahren
Zeit_PZ  = toc;
tic;
erg_EH   = ODE_EH(f, t0, T, x0, dt_EH); # Euler-Heun
Zeit_EH  = toc;

## Ausgabe und Druck der Ergebnisse
clf;
h = figure(1);
set (h,'papersize',[15 5])
set (h, 'paperposition', [0,0,[15 5]]);
  subplot(1,2,1)
    plot(erg_PZ(1,:),erg_PZ(2,:),'linewidth',3,'color','red')
    hold on
    plot(tte,x(tte),'linewidth',2,'color',[.5 .5 .5]);
    xlabel('t'), ylabel('x(t)')
    title(['Polygonzugverfahren mit dt = ', num2str(dt_PZ);...
            'N = ', num2str(length(erg_PZ(1,:))),' Schritte'; 
            'Laufzeit = ',num2str(Zeit_PZ), ' Sekunden'])
    set(gca,'linewidth',2,'fontsize',22)

  subplot(1,2,2)
    plot(erg_EH(1,:),erg_EH(2,:),'linewidth',3,'color','red')
    hold on
    plot(tte,x(tte),'linewidth',2,'color',[.5 .5 .5]);
    xlabel('t'), ylabel('x(t)')
    title(['Polygonzugverfahren mit dt = ', num2str(dt_EH);...
            'N = ', num2str(length(erg_EH(1,:))),' Schritte'; 
            'Laufzeit = ',num2str(Zeit_EH), ' Sekunden'])
    set(gca,'linewidth',2,'fontsize',22)
hold off