## Mathematikinformation Nr. 71 / 72
## http://mathematikinformation.info/
##
## Lei Baozhen, Beijing Union University
## Harald Loewe, Technische Universitaet Braunschweig
## Numerische Verfahren zur Loesung von Differentialgleichungen
##
##############################################################
### Zu Aufgabe 2.2.4 und 2.2.5: Euler-Heun-Verfahren       ###
##############################################################
##
## Ver. 2019-12-15
## Kodierung in UTF-8; Octave-Version 5.10

##############################################################
### Das Script braucht eine laengere Zeit zur Ausfuehrung  ###
##############################################################


## Aufraeumen
clear -a;
clf;

## Initialisierung Figure-Umgebung
fig = figure(1);
  groesse = [25 25];
  set (fig,'papersize',groesse)
  set (fig, 'paperposition', [0,0,groesse]);

## Wir fuehren die Verfahren
## - Polygonzugverfahren
## - Euler-Heun-Verfahren
## - kombiniertes Euler-Heun-Verfahren / Polygonzugverfahren
## anhand der Anfangswertprobleme
## - Logistisches Wachstum
## - Zerfall von Radon
## - Differentialgleichung aus Aufgabe 1.6.5
## aus. Die Ergebnisse werden in einem Plot mit 3x3 Subplots dargestellt.

## Logistisches Wachstum
## Anfangswertproblem
r     = 0.0017;       # Wachstumsrate
K     = 10000;        # Grenzbestand
t0    = 0;            # Intervallanfang
T     = 2;            # Intervallende
f     = @(t,x) r*x*(K-x); # rechte Seite der Differentialgleichung
x0    = 3;            # Anfangsbestand
#################################################
## Verfahrensparameter                         ##
#################################################
alf   = 10^-5;
sig   = 0.9;
## Verfahren
tic;  
erg_a   = ODE_PZSW(f, t0, T, x0,{'sigma',0.9','alpha',alf});
tta     = erg_a(1,:);
anza    = length(tta);
stpa    = tta(2:anza)-tta(1:anza-1);
mina    = min(stpa);
maxa    = max(stpa);
Zeit_a  = toc;
tic
erg_b   = ODE_EHSW(f, t0, T, x0,{'sigma',0.9','alpha',alf});
ttb     = erg_b(1,:);
anzb    = length(ttb);
stpb    = ttb(2:anzb)-ttb(1:anzb-1);
minb    = min(stpb);
maxb    = max(stpb);
Zeit_b  = toc;
tic
erg_c   = ODE_EHvsPZ(f, t0, T, x0,{'sigma',0.9','alpha',alf});
ttc     = erg_c(1,:);
anzc    = length(ttc);
stpc    = ttc(2:anzc)-ttc(1:anzc-1);
minc    = min(stpc);
maxc    = max(stpc);
Zeit_c  = toc;
tic
## Ausgabe
subplot(3,3,1)
    plot(tta,erg_a(2,:),'linewidth',2,'color','blue')
    xlabel('t'), ylabel('x(t)')
    title(['Log. W. / PZSW; \alpha = ',num2str(alf);... 
            num2str(anza), ' Schritte';...
            num2str(mina),'...',num2str(maxa);...
            'Laufzeit: ',num2str(Zeit_a), ' Sekunden'])
    set(gca,'linewidth',2,'fontsize',18)
  subplot(3,3,2)
    plot(ttb,erg_b(2,:),'linewidth',2,'color','red')
    xlabel('t'), ylabel('x(t)')
    title(['Log. W. / EHSW; \alpha = ',num2str(alf);... 
            num2str(anzb), ' Schritte';...
            num2str(minb),'...',num2str(maxb);...
            'Laufzeit: ',num2str(Zeit_b), ' Sekunden'],'fontsize',20)
    set(gca,'linewidth',2,'fontsize',18)
  subplot(3,3,3)
    plot(ttc,erg_c(2,:),'linewidth',2,'color','green')
    xlabel('t'), ylabel('x(t)')
    title(['Log. W. / EHvsPZ; \alpha = ',num2str(alf);... 
            num2str(anzc), ' Schritte';...
            num2str(minc),'...',num2str(maxc);...
            'Laufzeit: ',num2str(Zeit_c), ' Sekunden'])
    set(gca,'linewidth',2,'fontsize',18)
    
## Zerfall von Radon
## Anfangswertproblem fuer den Zerfall von Radon:
kp  = -0.1824;              # Zerfallskonstante
f   = @(t,x) kp*x;          # rechte Seite
t0  = 0;                    # Intervall-Anfang
T   = 365;                  # Intervall-Ende (365 Tage)
x0  = 10^3;                 # Anfangswert    (1000 kg)
#################################################
## Verfahrensparameter                         ##
#################################################
alf   = 10^-6;
sig   = 0.9;
## Verfahren
tic;  
erg_a   = ODE_PZSW(f, t0, T, x0,{'sigma',0.9','alpha',alf});
tta     = erg_a(1,:);
anza    = length(tta);
stpa    = tta(2:anza)-tta(1:anza-1);
mina    = min(stpa);
maxa    = max(stpa);
Zeit_a  = toc;
tic
erg_b   = ODE_EHSW(f, t0, T, x0,{'sigma',0.9','alpha',alf});
ttb     = erg_b(1,:);
anzb    = length(ttb);
stpb    = ttb(2:anzb)-ttb(1:anzb-1);
minb    = min(stpb);
maxb    = max(stpb);
Zeit_b  = toc;
tic
erg_c   = ODE_EHvsPZ(f, t0, T, x0,{'sigma',0.9','alpha',alf});
ttc     = erg_c(1,:);
anzc    = length(ttc);
stpc    = ttc(2:anzc)-ttc(1:anzc-1);
minc    = min(stpc);
maxc    = max(stpc);
Zeit_c  = toc;
tic
## Ausgabe
subplot(3,3,4)
    plot(tta,erg_a(2,:),'linewidth',2,'color','blue')
    xlabel('t'), ylabel('x(t)')
    title(['Radon / PZSW; \alpha = ',num2str(alf);... 
            num2str(anza), ' Schritte';...
            num2str(mina),'...',num2str(maxa);...
            'Laufzeit: ',num2str(Zeit_a), ' Sekunden'])
    set(gca,'linewidth',2,'fontsize',18)
  subplot(3,3,5)
    plot(ttb,erg_b(2,:),'linewidth',2,'color','red')
    xlabel('t'), ylabel('x(t)')
    title(['Radon / EHSW; \alpha = ',num2str(alf);... 
            num2str(anzb), ' Schritte';...
            num2str(minb),'...',num2str(maxb);...
            'Laufzeit: ',num2str(Zeit_b), ' Sekunden'],'fontsize',20)
    set(gca,'linewidth',2,'fontsize',18)
  subplot(3,3,6)
    plot(ttc,erg_c(2,:),'linewidth',2,'color','green')
    xlabel('t'), ylabel('x(t)')
    title(['Radon / EHvsPZ; \alpha = ',num2str(alf);... 
            num2str(anzc), ' Schritte';...
            num2str(minc),'...',num2str(maxc);...
            'Laufzeit: ',num2str(Zeit_c), ' Sekunden'])
    set(gca,'linewidth',2,'fontsize',18)


## DGL aus Aufgabe 1.6.5
## Initialisierung des Abfabgswertproblems
f   = @(t,x) -50*x+50*cos(t)-sin(t);
x0  = 3;                   # Anfangswert
t0  = 0;                   # Intervallanfang
T   = 10;                  # Intervallende
#################################################
## Verfahrensparameter                         ##
#################################################
alf   = 10^-7;
sig   = 0.9;
## Verfahren
tic;  
erg_a   = ODE_PZSW(f, t0, T, x0,{'sigma',0.9','alpha',alf});
tta     = erg_a(1,:);
anza    = length(tta);
stpa    = tta(2:anza)-tta(1:anza-1);
mina    = min(stpa);
maxa    = max(stpa);
Zeit_a  = toc;
tic
erg_b   = ODE_EHSW(f, t0, T, x0,{'sigma',0.9','alpha',alf});
ttb     = erg_b(1,:);
anzb    = length(ttb);
stpb    = ttb(2:anzb)-ttb(1:anzb-1);
minb    = min(stpb);
maxb    = max(stpb);
Zeit_b  = toc;
tic
erg_c   = ODE_EHvsPZ(f, t0, T, x0,{'sigma',0.9','alpha',alf});
ttc     = erg_c(1,:);
anzc    = length(ttc);
stpc    = ttc(2:anzc)-ttc(1:anzc-1);
minc    = min(stpc);
maxc    = max(stpc);
Zeit_c  = toc;
tic
## Ausgabe
subplot(3,3,7)
    plot(tta,erg_a(2,:),'linewidth',2,'color','blue')
    xlabel('t'), ylabel('x(t)')
    title(['A.1.6.5 / PZSW; \alpha = ',num2str(alf);... 
            num2str(anza), ' Schritte';...
            num2str(mina),'...',num2str(maxa);...
            'Laufzeit: ',num2str(Zeit_a), ' Sekunden'])
    set(gca,'linewidth',2,'fontsize',18)
  subplot(3,3,8)
    plot(ttb,erg_b(2,:),'linewidth',2,'color','red')
    xlabel('t'), ylabel('x(t)')
    title(['A.1.6.5 / EHSW; \alpha = ',num2str(alf);... 
            num2str(anzb), ' Schritte';...
            num2str(minb),'...',num2str(maxb);...
            'Laufzeit: ',num2str(Zeit_b), ' Sekunden'],'fontsize',20)
    set(gca,'linewidth',2,'fontsize',18)
  subplot(3,3,9)
    plot(ttc,erg_c(2,:),'linewidth',2,'color','green')
    xlabel('t'), ylabel('x(t)')
    title(['A.1.6.5 / EHvsPZ; \alpha = ',num2str(alf);... 
            num2str(anzc), ' Schritte';...
            num2str(minc),'...',num2str(maxc);...
            'Laufzeit: ',num2str(Zeit_c), ' Sekunden'])
    set(gca,'linewidth',2,'fontsize',18)    


# Druck in Datei    
print ("-r600", "-painters", ['Aufgabe_2_2_4u5.png']);