## Mathematikinformation Nr. 71 / 72
## http://mathematikinformation.info/
##
## Lei Baozhen, Beijing Union University
## Harald Loewe, Technische Universitaet Braunschweig
## Numerische Verfahren zur Loesung von Differentialgleichungen
##
##############################################################
### Zu Aufgabe 2.4.3 (Fadenpendel)                         ###
##############################################################
##
## Ver. 2019-12-15
## Kodierung in UTF-8; Octave-Version 5.10

## Initialisierung des Anfangswertproblems
AWP_init_FP;

## Zur Bestimmung der Periodenlaengen benoetigen wir das Signalpaket
pkg load signal;

## Liste der Anfangsauslenkungen
amp_list = [pi/8 pi/4 pi/2-0.1];
num_amp = length(amp_list);

fig = figure(1);
groesse = [25 6];
set (fig,'papersize',groesse)
set (fig, 'paperposition', [0,0,groesse]);

for lauf = 1:num_amp
  amp = amp_list(lauf);
  x0  = [amp, 0];
  ## Durchfuehrung eines passenden Verfahrens 
  ###################################################################
  ## Sowohl ODE_PZSW als auch ODE_EHvsPZ liefern bei alpha = 10^-4 ##
  ## falsche Loesungen, da die Maximalausschlaege des Pendels      ##
  ## wachsen - auch bei Steuerung der Schrittweite muss man daher  ##
  ## noch aufpassen!                                               ##
  ###################################################################
  erg = ODE_RKSW(f, t0, T, x0, {'alpha',10^-4});
  ttt = erg(1,:);
  ## Fuer grosse Auslenkungen muss das Ergebnis modulo 2 pi
  ## reduziert werden:
  xxx = mod(pi+erg(2,:),2*pi)-pi;
  
  ## Wir bestimmen die lokalen Extrema in der Liste xxx
  ## sowie deren Positionen; hierzu gibt es die Funktion
  ## "findpeaks" aus dem Paket "signal"
  [pks pos] = findpeaks(xxx,"DoubleSided");
  ## In der Liste pos befinden sich die Positionen der lokalen
  ## Extrema der Liste xxx. Die zugehoerigen Stellen befinden sich
  ## an den gleichen Positionen in der Liste ttt:
  ttt_pks   = ttt(pos);
  ## Die Differenzen zwischen aufeinanderfolgenden Extremalstellen
  ## ist die halbe Periodenlaenge
  anz_pks   = length(ttt_pks);
  prd_list  = ttt_pks(2:anz_pks) - ttt_pks(1:anz_pks-1);
  ## Als Schaetzung nehmen wir den Median, um eventuelle Ausreisser 
  ## loszuwerden. Diese Schaetzung muessen wir noch verdoppeln!
  prd       = 2*median(prd_list);
  ## Dieses Ergebnis tragen wir in die Ueberschrift der Graphik ein

  ## Plot der Ergebnisse als subfigure
  subplot(1, num_amp, lauf)
    ## Plot der Naeherungsloesung der Physiker in Grau
    plot(ttt,amp*cos(ttt),'linewidth',2,'color',[.5 .5 .5])
    hold on                 # naechste Grafik dazufuegen
    ## Plot des Polygonzugs
    plot(ttt,xxx,'linewidth',3,'color','blue')
    ## Plot der Extrema
    plot(ttt_pks,pks,'o','linewidth',3,'color','blue')
    ## Legende, Label etc.
    xlabel('t'), ylabel('Auslenkung [rad]'), axis([t0 T -1.1*amp 1.1*amp])
    title(['Auslenkung = ', num2str(amp);'Periode = ',num2str(prd);...
           num2str(length(xxx)),' Schritte'])
    set(gca,'linewidth',2,'fontsize',18)
    hold off
endfor

## Abspeichern als Bild in akzeptabler Aufloesung.

## Umwandeln von amp_list in einen String fuer den Dateinamen
amp_str = '';
for lauf = 1:num_amp
  amp_str = [amp_str,' (',num2str(lauf),') ',num2str(amp_list(lauf))];
endfor  

## Ausgabe
## Falls die Marker keine Kreise sind: "-painters" als Option einfuegen!
print ("-r600", "-painters", ["Fadenpendel_phi0=",amp_str,".png"]);